function [FinalMask]=PodoFind(ImageIn,LowerDiameter,UpperDiameter,FeatureSize,PixelSize,RoundnessThreshold,DispFig)
% PODOFIND segments podosome cores in fluorescence microscopy images of
% actin stained cells
%
% SYNOPSIS
% [FinalMask]=PodoFind(ImageIn,LowerDiameter,UpperDiameter,FeatureSize,... 
% PixelSize,RoundnessThreshold,DispFig)
%
% INPUT
% ImageIn =         Fluorescence microscopy image of cells stained for actin
% LowerDiameter =   Diameter of smallest objects to be identified as podosome
%                   cores in um
% UpperDiameter =   Diameter of largest objects to be identified as podosome
%                   cores in um
% FeatureSize =     Average diameter of podosome cores in um
% PixelSize =       Pixelsize in um
% RoundnessThreshold = Threshold for roundness of objects to be identified
%                   as podosome cores 
%                   (Roundness of an ellipse = Short Diameter / Long Diameter)
% DispFig =         1 if images of processing steps should be displayed, 0
%                   otherwise
%
% OUTPUT
% FinalMask = Binary image containing the segmented podosome cores
%
% Created by Marjolein Meddens, April 2012


% Calculate Algorithm parameters
SizeLimitLow = round(pi*(((LowerDiameter/PixelSize)*0.5)^2));
SizeLimitHigh = round(pi*(((UpperDiameter/PixelSize)*0.5)^2));
SizeLimitLowOutliers = round(SizeLimitLow/1.5);
SizeLimitHighOutliers = round(SizeLimitHigh*1.5);

FilterSizeMaskImage = 1;
FilterSizeThresholdImage = 0.5;
LocalThresholdRadius = round((FeatureSize/PixelSize)*1.4);
FilterSizeMedif = 3;

ImageWidth = size(ImageIn,1);
ImageHeight= size(ImageIn,2);

% Gaussian smoothing
Image1 = gaussf(ImageIn, 0.5);
% Unsharp mask
Image1 = Image1 - laplace(Image1,FilterSizeMaskImage);

% Display Figure
if DispFig
dipfig('Image1',[10,400,ImageWidth,ImageHeight]);
dipshow(Image1,'notruesize');
end

% Calculate local threshold
mean1 = unif(1*Image1,LocalThresholdRadius,'elliptic');
var1  = varif(1*Image1,LocalThresholdRadius,'elliptic');
LocalThreshold = mean1 + (0.5 * (var1^0.5));

% Apply local threshold
Mask1 = round(Image1 > LocalThreshold);

% Filter on size and roundness
Mask1 = label(Mask1,1,SizeLimitLowOutliers,SizeLimitHighOutliers);
msr1 = measure(Mask1,[],{'Size','Feret'},[],1,0,0);
SizeMask = msr2obj(Mask1,msr1,'Size',1);
maxFeret = msr2obj(Mask1,msr1,'Feret',1);
Roundness = 4 * SizeMask./(pi*maxFeret.^2);
Mask1 = Roundness > RoundnessThreshold*1;

% Display Figure
if DispFig
dipfig('Mask1',[220,380,ImageWidth,ImageHeight]);
dipshow(Mask1,'Labels','notruesize');
end

% Gaussian smoothing
Image2 = gaussf(ImageIn, FilterSizeThresholdImage);

% Display Figure
if DispFig
dipfig('Image2',[430,360,ImageWidth,ImageHeight]);
dipshow(Image2,'notruesize');
end

% Combine two images
Image3 = (Mask1>0) * Image2;

% Median filter
Image3 = medif(Image3,FilterSizeMedif,'elliptic');

% Isodata threshold
[Mask2, values] = dip_isodatathreshold(Image3, (Image3>0), 2);
Mask2=Mask2>0;

% Display Figure
if DispFig
dipfig('Mask2',[640,340,ImageWidth,ImageHeight]);
dipshow(Mask2,'lin','notruesize');
end

%  Distance transform based watershed + filter on size
Dis = dt(Mask2>0,1,'true');
Was = watershed(-Dis,2);
FinalMask = label( (~Was)&Mask2>0 ,1,SizeLimitLow,SizeLimitHigh);

% Display Figure
if DispFig
Overlay = overlay((ImageIn/max(ImageIn))*255,label(FinalMask>0));
dipfig('Overlay',[850,300,ImageWidth,ImageHeight]);
dipshow(Overlay,'notruesize');
end
end