% RINGMEASURE measures intensity in dilated and eroded podosome core ROIs
%
% SYNOPSIS
% [RingData,ColumnHeadings] = RingMeasure(GreyImage,RoiImage,NumberErosion,NumberDilation,Cumulative,PerPod,PixelSize,UpperDiameter)
%
% INPUT
% GreyImage =       Image in which ring intensities will be measured
% RoiImage =        Binary image containing segmented podosome cores
% NumberErosion =   Number of pixels the core should be eroded
% NumberDilation =  Number of pixels the core should be dilated
% Cumulative =      1 if all dilations and erosions should be taken
%                   together, 0 if intensity should be calculated for each
%                   pixel distance separately
% PerPod =          1 if the whole image should be taken together, 0 if
%                   intensities should be calculated for each podosome
%                   separately
% PixelSize =       Pixelsize in um (to calculate crop size)
% UpperDiameter =   Diameter of largest objects considered podosomes (to
%                   calculate crop size)
%
% OUTPUT
% RingData =        Structure containing mean gray values for chosen 
%                   dilations and erosions
% ColumnHeadings =  Column headings of RingData 

% Created by Marjolein Meddens, April 2012

function [RingData,ColumnHeadings] = RingMeasure(GreyImage,RoiImage,NumberErosion,NumberDilation,Cumulative,PerPod,PixelSize,UpperDiameter)

% Initialization
RoiImage = RoiImage>0;
RoiImageLabel = label(RoiImage);
ColumnHeadings = cell(1,NumberErosion+NumberDilation);

% Create cell array containing column headings and an output array
if Cumulative
    ColumnHeadings=[];
    if PerPod
        out = zeros(max(RoiImageLabel), 1);
    end
else
    for k = 1:NumberErosion
        ColumnHeadings(1,k) = {['Core ' num2str(k-(NumberErosion+1))]};
    end
    for m = 1:NumberDilation
        ColumnHeadings(1,(NumberErosion+m)) = {['Core +' num2str(m)]};
    end
    if PerPod
        out = zeros(max(RoiImageLabel), NumberErosion+NumberDilation);
    end
end


%% Calculation

msr = measure(RoiImageLabel,[],{'Center'},[],Inf,0,0);
DTdilation = dt(~RoiImage);
DTerosion = dt(RoiImage);
Limit = UpperDiameter/PixelSize;

if PerPod
    for ii = 1:max(RoiImageLabel)
        % Determine coordinates for cropping
        Xcenter = round(msr(ii).Center(1));
        Ycenter = round(msr(ii).Center(2));
        Xmin = Xcenter-Limit-NumberDilation;
        if Xmin<0
            Xmin=0;
        end
        Xmax = Xcenter+Limit+NumberDilation;
        if Xmax>size(RoiImage,1)-1
            Xmax = size(RoiImage,1)-1;
        end
        Ymin = Ycenter-Limit-NumberDilation;
        if Ymin<0
            Ymin=0;
        end
        Ymax = Ycenter+Limit+NumberDilation;
        if Ymax>size(RoiImage,2)-1
            Ymax = size(RoiImage,2)-1;
        end
        
        % make crops for RoiImage and DT
        Crop = RoiImageLabel(Xmin:Xmax,Ymin:Ymax)==ii;
        DTcrop = dt(~Crop);
        mask = DTdilation(Xmin:Xmax,Ymin:Ymax) == DTcrop;
        DTfinalD = mask*DTcrop;
        DTfinalE = DTerosion(Xmin:Xmax,Ymin:Ymax)*Crop;
        
        % Calculate mean intensities using distance maps masks
        if Cumulative
            mE = (DTfinalE <= NumberErosion) & (DTfinalE > 0);
            mD = (DTfinalD <= NumberDilation) & (DTfinalD > 0);
            out(ii,1) = mean(GreyImage(Xmin:Xmax,Ymin:Ymax),mE+mD);
        else
            for kk=1:NumberErosion
                m = (DTfinalE <= kk) & (DTfinalE > kk-1);
                out(ii,NumberErosion-(kk-1)) = mean(GreyImage(Xmin:Xmax,Ymin:Ymax),m);
            end
            for jj=1:NumberDilation
                m = (DTfinalD <= jj) & (DTfinalD > jj-1);
                out(ii,NumberErosion+jj) = mean(GreyImage(Xmin:Xmax,Ymin:Ymax),m);
            end
        end
    end
else
    if Cumulative
        mE = (DTerosion <= NumberErosion) & (DTerosion > 0);
        mD = (DTdilation<= NumberDilation) & (DTdilation > 0);
        out = mean(GreyImage,mE+mD);
    else
        out = zeros(1, NumberErosion+NumberDilation);
        for kk=1:NumberErosion
            m = (DTerosion <= kk) & (DTerosion > kk-1);
            out(1,NumberErosion-(kk-1)) = mean(GreyImage,m);
        end
        for jj=1:NumberDilation
            m = (DTdilation <= jj) & (DTdilation > jj-1);
            out(1,NumberErosion+jj) = mean(GreyImage,m);
        end
    end
end
RingData = out;

